<?php

//缓存类,目前设定了file、redis缓存，默认是file缓存

namespace blockphp\lib;

use blockphp\lib\cache\Driver;

class Cache
{

    public static $instance = null;
    public static $options = null;
    /*
     * 读取缓存配置
     */
    public static function getConfig(){
        $globleConfig  = config('cache');
        $appConfig = config('cache',APP_PATH.'/'.$GLOBALS['module'].'/config.php');
        $options = array_merge($globleConfig,$appConfig);
        self::$options = $options;
        return $options;
    }

    /*
     * 连接缓存驱动
     * @param array $config 缓存配置
     */
    public static function connect($options=null)
    {
        if(empty($options)){
            $type = self::getConfig()['type'];
        }else{
            self::$options = $options;
            $type = $options['type'];
        }

        if($type == 'File'){
            $instance =  \blockphp\lib\cache\driver\File::getInstant();
        }elseif($type == 'Redis'){
            $instance= \blockphp\lib\cache\driver\Redis::getInstant();
        }
        self::$instance = $instance;
    }

    /*
     * 写入缓存
     * @access public
     * @param  string   $name   缓存标识
     * @param  mixed    $value  存储数据
     * @param  int|null $expire 有效时间 0为永久
     * @return boolean
     */
    public static function set($name,$value,$expire =null)
    {
        if(empty(self::$instance)){
            self::connect();
        }
        return self::$instance->set($name,$value,$expire,self::$options);
    }

    /*
     * 自定义缓存标签
     * @param  string   $tag
     */
    public static function tag($tag)
    {
        if(empty(self::$instance)){
            self::connect();
        }
        $options = self::$options;
        $options['tag'] = $tag;
        self::$options = $options ;
        return new self;
    }


    /*
     * 读取缓存
     * @access public
     * @param  string   $name   缓存标识
     */
    public static function get($name)
    {
        if(empty(self::$instance)){
            self::connect();
        }
        return self::$instance->get($name,self::$options);
    }

    /*
     * 删除缓存
     * @access public
     * @param  string   $name   缓存标识
     * @return bool
     */
    public static function rm($name)
    {
        if(empty(self::$instance)){
            self::connect();
        }
        return self::$instance->rm($name,self::$options);
    }

    /*
     * 获取u并删除缓存
     * @access public
     * @param  string   $name   缓存标识
     * @return bool
     */
    public static function pull($name)
    {
        if(empty(self::$instance)){
            self::connect();
        }
        return self::$instance->pull($name,self::$options);
    }


    /*
    * 获缓存自增
    * @param string $name 缓存名
     *@param  int  $step 步长
    * @param array  $option 缓存配置
    */
    public static function inc($name,$step=1){
        if(empty(self::$instance)){
            self::connect();
        }
        return self::$instance->autoNum($name,$step,self::$options);
    }

    /*
     * 获缓存自减
     * @param string $name 缓存名
      *@param  int  $step 步长
     * @param array  $option 缓存配置
     */
    public static function dec($name,$step=1){
        $step = $step*-1;
        if(empty(self::$instance)){
            self::connect();
        }
        return self::$instance->autoNum($name,$step,self::$options);
    }



}