<?php
/*
 * 分页类
 */
namespace blockphp\lib;
class Paginate
{
    public $param= null; //页面显示数据条数
    public $pageNum = null; //当前页面
    public $pageData = array(); //分页数据
    public $count=null; //数据总条数
    public $config = null; //分页配置
    public $pageCount; //共几页



    /*
     * 初始化分页参数
     * @param  $param  int 分页条数
     * @param  $config  array 分页配置
     * @param $pageData array 分页数据
     * @param $count int  数据总条数
     */
    public function paginate($param,$config,$pageData,$count)
    {

        $this->param = $param;

        //用户自定义配置高于默认配置
        if(is_array($config)){
            $this->config =array_merge($this->config(),$config);
        }else{
            //用户cofing.php自定义配置高于默认配置
            $this->config =array_merge($this->config(),config('paginate'));
        }
		
		  
        $this->pageNum = input('get.page');
        $this->pageData = $pageData;
        $this->count = $count;
        $this->pageCount = ceil($this->count/$this->param);
        return $this;
    }

    //默认分页配置
    public function config()
    {
        return [
            'list_rows' => 8,            //分页数字的长度
            'prveText'  => '上一页',     //上一页文本
            'nextText'  => '上一页',     //下一页文本
            'ellipsis'  => '...',       //缩略字符
            'simple' => false           //简单模式
        ];
    }


    public function render()
    {
		 
		 
		 //当数据条数少分页数时不分页
		 if($this->count <= $this->param){
			 return false;
		 }
		 
        //限制页码范围
        if($this->pageNum < 1){
            $this->pageNum =1;
        }
        if($this->pageNum > $this->pageCount){
            $this->pageNum = $this->pageCount;
        }


        //带参URL
        $baseUrl = preg_replace('/\?page=[0-9]*&?/','',$_SERVER['REQUEST_URI']);

        if(preg_match('/\?/',$baseUrl)){
            $baseUrl = $baseUrl.'&page=';
        }else{
            $baseUrl = $baseUrl.'?page=';
        }


        //上一页按钮的状态
        $this->pageNum > 1? $prevStatus ='': $prevStatus ='disabled';
        //下一页按钮的状态
        $this->pageNum < $this->pageCount? $nextStatus ='': $nextStatus ='disabled';

        // 上一页 html
        if($this->pageNum>1){
            $prevHtml = '<span class="prev_page '.$prevStatus.'"><a href="'.$baseUrl.($this->pageNum-1).'">'.$this->config['prveText'].'</a></span>';
        }else{
            $prevHtml = '<span class="prev_page '.$prevStatus.'">'.$this->config['prveText'].'</span>';
        }

        // 下一页 html
        if($this->pageNum < $this->pageCount){
            $nextHtml = '<span class="next_page '.$nextStatus.'"><a href="'.$baseUrl.($this->pageNum+1).'">'.$this->config['nextText'].'</a></span>';
        }else{
            $nextHtml = '<span class="next_page '.$nextStatus.'">'.$this->config['nextText'].'</span>';
        }




        //第一页码 html
        $this->pageNum == 1? $minNum = '<a class="active" href="'.$baseUrl.'1">1</a>':$minNum = '<a href="'.$baseUrl.'1">1</a>';
        //最大页码 html
        $this->pageNum == $this->pageCount ? $maxNum = '<a class="active" href="'.$baseUrl.$this->pageCount.'">'.$this->pageCount.'</a>': $maxNum = '<a href="'.$baseUrl.$this->pageCount.'">'.$this->pageCount.'</a>';




        //计算当前页，左右页码
        $rl_len = ceil(($this->config['list_rows']-2)/2);

        //判断左右省略号是否显示
        $rel = $lel = '';
        if($this->pageNum-$rl_len > $rl_len){
            $lel = '<a class="ellipsis" href="javasript:void(0);">'.$this->config['ellipsis'].'</a>';
        }
        if($this->pageCount-$this->pageNum-1 > $rl_len){
            $rel = '<a class="ellipsis" href="javasript:void(0);">'.$this->config['ellipsis'].'</a>';
        }

        //当总页数小于配置页码长度时，不需要省略图
        if($this->pageCount <= $this->config['list_rows']){
            $num_list = ''; //页码html
            for($i=1;$i<=$this->pageCount;$i++){
                if($i!=1 && $i!=$this->pageCount){
                    if($this->pageNum == $i){
                        $num_list .= '<a class="active" href="'.$baseUrl.$i.'">'.$i.'</a>';
                    }else{
                        $num_list .= '<a href="'.$baseUrl.$i.'">'.$i.'</a>';
                    }
                }
            }
            $num_list = '<span class="num_list">'.$minNum.$num_list.$maxNum.'</span>';
        }else{
            //当前页左边页码
            $leftList = '';
            for($i=$rl_len;$i>0;$i--){
                if($this->pageNum-$i > 1){
                    $num = $this->pageNum-$i;
                    $leftList .= '<a href="'.$baseUrl.$num.'">'.$num.'</a>';
                }
            }

            //当前页页码
            $nowPage = '';
            if($this->pageNum != 1 && $this->pageNum!=$this->pageCount ){
                $nowPage = '<a class="active" href="javascript:void(0);">'.$this->pageNum.'</a>';
            }


            //当前页右边页码
            $rightList = '';
            for($i=1;$i<=$rl_len;$i++){
                if($this->pageNum+$i < $this->pageCount ){
                    $num = $this->pageNum+$i;
                    $rightList .= '<a href="'.$baseUrl.$num.'">'.$num.'</a>';
                }
            }




            $num_list ='<span class="num_list">'.$minNum.$lel.$leftList.$nowPage.$rightList.$rel.$maxNum.'</span>';
        }


        //页码 html
        if($this->config['simple']){  //简单模式
             $listHtml ='<div class="pagination">'.$prevHtml.$nextHtml.'</div>';
            return  $listHtml;
        }
        $listHtml ='<div class="pagination">'.$prevHtml.$num_list.$nextHtml.'</div>';
        return  $listHtml;

    }


}