<?php
namespace blockphp\lib\cache\driver;

class File
{
    //1.静态私有属性，保存当前类的实例
    private static $instance = null;


    //2.生成当前类的唯一实例
    public static function getInstant()
    {
        //如果不是当前的实例，那么实例化当前类创建新实例
        if(!self::$instance instanceof self){
            self::$instance = new self();
        }
        return self::$instance;
    }


    public  function set($name,$value,$expire = null,$options)
    {
        empty($expire)? $expire = $options['expire'] : null;

        $fileName = $this->setKey($name,$options);
        $data = serialize($value);  //序例化数据
        $data = "<?php\n//" . sprintf('%012d', $expire) . "\n exit();?>\n" . $data;
        file_put_contents($fileName,$data);
        return true;
    }

    /*
     * 生成缓存唯一标识和文件夹
     * @param string $name
     * @param array  $option 缓存配置
     * @return string 缓存文件名
     */
    public function setKey($name,$options)
    {
        //$name md5 加密，取字符串前2位为文件夹名，后30位为缓存文件名
        if(isset($options['tag'])){
            //当存在tag标签存在
            $name ='tag_'.$name;
        }
        $name = md5($name);
        empty($options['prefix']) ? null : $options['prefix'] .= '/';
        $fileFolder = $options['path'].'/'.$options['prefix'].substr($name,0,2);

        //当文件夹不存在时生成
        is_dir($fileFolder)?null:mkdir($fileFolder,0777,true);
        $fileName =$fileFolder.'/'.substr($name,2).'.php';

        //如果存在tag标签，则缓存记录
        if(isset($options['tag'])){
            $this->tag($options['tag'],$fileName,$options);
        }
        return $fileName;
    }

    /*
     * 生成tag记录缓存
     * @param string $tagName 标签名
     * @param string $data 被缓存的文件名
     * @param array  $option 缓存配置
     */
    public function tag($tagName,$data,$options)
    {
        //$name md5 加密，取字符串前2位为文件夹名，后30位为缓存文件名
        $tagName = md5($tagName);
        empty($options['prefix']) ? null : $options['prefix'] .= '/';
        $fileFolder = $options['path'].'/'.$options['prefix'].substr($tagName,0,2);

        //当文件夹不存在时生成
        is_dir($fileFolder)?null:mkdir($fileFolder,0777,true);
        $fileName =$fileFolder.'/'.substr($tagName,2).'.php';

        //将缓存文件名追加到tag缓存文件
        is_file($fileName)?$origin = file_get_contents($fileName):$origin ='';
        $data = explode(',',$origin.','.$data);
        $data = array_unique($data);
        $data = implode(',',$data);
        file_put_contents($fileName,$data);
    }


    /*
    * 读取缓存
    * @param string $tagName 缓存名
    * @param array  $option 缓存配置
    */
    public function get($name,$options)
    {
        //当存在tag标签存在
        if(isset($options['tag'])){
            $name ='tag_'.$name;
        }
        $name = md5($name);
        empty($options['prefix']) ? null : $options['prefix'] .= '/';
        $fileName = $options['path'].'/'.$options['prefix'].substr($name,0,2).'/'.substr($name,2).'.php';

        if(!is_file($fileName)){
            return false;
        }
        $content = file_get_contents($fileName); //获取所有缓存内容
        $expire = (int) substr($content, 8, 12); //读取缓存时间
        $data = substr($content, 32); //读取缓存数据
        $data = unserialize($data);

        //判断缓存是否过期，如果过期就删除缓存文件，否则就返回缓存数据
        if($expire > 0){
            if(time() < (filemtime($fileName) + $expire)){
                return $data;
            }else{
                unlink($fileName);
                return false;
            }
        }else{
            return $data;
        }
    }


    /*
    * 删除缓存
    * @param string $tagName 标签名
    * @param string $data 被缓存的文件名
    * @param array  $option 缓存配置
    */
    public function rm($name,$options)
    {
        //缓存前缀
        empty($options['prefix']) ? null : $options['prefix'] .= '/';

        //当存在tag标签存在
        if(isset($options['tag'])){
            $name ='tag_'.$name;
        }

        $name = md5($name);
        empty($options['prefix']) ? null : $options['prefix'] .= '/';
        $fileName = $options['path'].'/'.$options['prefix'].substr($name,0,2).'/'.substr($name,2).'.php';

        //删除tag缓存记录
        if(isset($options['tag'])){
            $tagName = md5($options['tag']);
            $tagFile= $options['path'].'/'.$options['prefix'].substr($tagName,0,2).'/'.substr($tagName,2).'.php';
            if(is_file($tagFile)){
                $content = file_get_contents($tagFile);
                $data  = str_replace(','.$fileName,'',$content);
                $count = count(explode(',',$data));
                if($count == 1){
                    unlink($tagFile);
                }else{
                    file_put_contents($tagFile,$data);
                }
            }
        }

        //删除缓存文件
        if(is_file($fileName)){
           return unlink($fileName);
        }

    }

    /*
    * 获取并删除缓存
    * @param string $name 缓存名
    * @param array  $option 缓存配置
    */
    public function pull($name,$option)
    {
       $data =  $this->get($name,$option);
       $this->rm($name,$option);
       return $data;
    }


    /*
    * 缓存自增或自减
    * @param string $name 缓存名
     *@param  int  $step 步长
    * @param array  $option 缓存配置
    */
    public function autoNum($name,$step,$options){
       $data = $this->get($name,$options);
       $value =  (int) $data + $step;
       $this->set($name,$value,$expire=null,$options);
       return $value;
    }


}