<?php
/*
 * 数据库连接,单例模式
 */

namespace blockphp\lib\db;
use PDO;
class Connection
{
    private  $dbConfig; //数据库的默认连接参数
    public $insertId = null; //新增主键id
    public $num = 0; //受影响的记录数量
    public $conn = null; //数据库的连接
    private static $instance = null; //单例模式,本类的实例

    /*
     * 构造方法
     * 私有化以防止外部实例化
     */
    function __construct()
    {
        //初始化连接参数
        $_DdConfig = require ROOT_PATH.'/config/dataBase.php';  //全局db配置参数
        $_appDbPath = APP_PATH.'/'.$GLOBALS['module'].'/'.'dataBase.php';
        is_file($_appDbPath)? $_appDbConfig = require $_appDbPath : $_appDbConfig = array();  //模型db配置参数
        $this->dbConfig = array_merge($_DdConfig,$_appDbConfig);

        //连接数据库
        $this->connect();

    }


    /*
     * @return Db|null
     * 获取当前类的单一实例
     */
    public static function getInstance()
    {
        if (!self::$instance instanceof self) {
            self::$instance = new self();
        }
        return self::$instance;
    }


    /*
     * 禁止外部克隆该实例
     */
    private function __clone()
    {
        // TODO: Implement __clone() method.
    }


    /*
     * 数据库的连接
     */
    private function connect()
    {
        try{
            //配置数据源DSN
            $dsn = "{$this->dbConfig['db']}:host={$this->dbConfig['host']};
            port={$this->dbConfig['port']};dbname={$this->dbConfig['dbname']};
            charset={$this->dbConfig['charset']}";

            //创建PDO对象
            $this->conn = new PDO($dsn,$this->dbConfig['user'],$this->dbConfig['pass']);

            //设置客户端的默认字符集
            $this->conn->query("SET NAMES {$this->dbConfig['charset']}");
            //设置为警告模式
            $this->conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);

        }catch (PDOException $e){
            die('数据库连接失败'.$e->getMessage());
        }
    }
    public function test(){
        return 'test';
    }


    /**
     * 调用Query类的查询方法
     * @access public
     * @param string    $method 方法名称
     * @param array     $args 调用参数
     * @return mixed
     */
    public function __call($method, $args)
    {
        return call_user_func_array([new Query($this->conn), $method], $args);
    }





}