<?php
/*
 * 数据库增删改查方法类
 */
namespace blockphp\lib\db;
use blockphp\lib\Cache;
use blockphp\lib\Db;
use blockphp\lib\Paginate;
use PDO;

class Query
{
    private $table = null;
    private $where = null;
    private $field = '*';
    private $limit = null;
    private $order = null;
    public $conn = null;
    public $insertId = null;
    public $alias = null;
    public $join = null;
    public $cacheData = null;
    public $cacheOption = array('switch'=>false,'name'=>'','time'=>0);

    //数据库连接对象
    function __construct($dbObj)
    {
        $this->conn = $dbObj;
    }

    //table 表名
    public function table($_table)
    {
        //数据库存前辍，默认为mk_
        $prefix = config('prefix','config/dataBase.php');
        $this->table = $prefix.$_table.' ';
        return $this;
    }


    //where条件
    public function where($param1,$param2=null,$param3=null)
    {
        //$param3为空，条件语句为 $param1 = $param2
        if(isset($param2) && !isset($param3)){
            $param2 = $this->varType($param2);
            $_where = ' AND '.$param1.'='.$param2;
        }

        //都不为空，条件语句为 $param1 $param2 $param3
        if(isset($param2) && isset($param3)){
            //判断 $param2 是否为关键字 IN，NOT IN ,BETWEEN,NOT BETWEEN
            $param2 = strtoupper($param2);
            if($param2 == 'IN'||$param2 == 'NOT IN'){
                if(!is_array($param3)){
                    $param3 = explode(',',$param3);
                }
                foreach ($param3 as $k3=>$v3) {
                    $param3[$k3] = $this->varType($v3); //为字符串加上双引号
                }
                $param3 = implode(',',$param3);
                $param3 = '('.$param3.')';
            }elseif($param2 == 'BETWEEN'||$param2 == 'NOT BETWEEN'){
                if(!is_array($param3)){
                    $param3 = explode(',',$param3);
                }
                foreach ($param3 as $k3=>$v3) {
                    $param3[$k3] = $this->varType($v3); //为字符串加上双引号
                }
                $param3 = $param3[0].' AND '.$param3[1];
            }else{
                $param3 = $this->varType($param3);
            }
            $_where = ' AND '.$param1.' '.$param2.' '.$param3;
        }

        //当$param2，$param3都为空
        if(!isset($param2) && !isset($param3)){
           if(is_array($param1)){
               $_where = '';
               foreach($param1 as $key=>$val){
                   if($key === 0){
                       exit('Error: where条件中键不能为 0');
                   }

                   //如值是数组
                   if(is_array($val)){
                       //判断 $val[0]是否为关键字 IN，NOT IN ,BETWEEN,NOT BETWEEN
                       $val[0] = strtoupper($val[0]);
                       //将数组值，中的字符串加上双引号
                       if(!is_array($val[1])){
                           $val[1] = explode(',',$val[1]);
                       }
                       foreach($val[1] as $vk=>$vv){
                           $val[1][$vk] = $this->varType($vv);
                       }

                       //判断 $val[0]是否为关键字 IN，NOT IN
                       if($val[0] == 'IN'||$val[0] == 'NOT IN'){
                           $val[1] = '('.implode(',',$val[1]).')';
                       }
                       //判断 $val[0]是否为关键字 BETWEEN,NOT BETWEEN
                       elseif($val[0] == 'BETWEEN'||$val[0] == 'NOT BETWEEN'){
                           $val[1] = $val[1][0].' AND '.$val[1][1];
                       }
                       //不是关键字，正常的数组条件
                       else{
                           $val[1] = $val[1][0];
                       }
                       $_where .= ' AND '.$key.' '.$val[0].' '.$val[1];
                   }else{
                       $_where .= ' AND '.$key.'='.$this->varType($val);
                   }
               }


           }else{
               echo 'error : where条件语句不正确';
           }
        }
        $this->where = $this->where.' WHERE '.substr($_where,4);
        return $this;
    }


    /*
     * join联表查询
     * $param1 string 数据表名 加别名
     * $param2 string 查询条件
     * $param3 string 默认INNER,可修改为LEFT或RIGHT或FULL
     */
    public function join($param1,$param2,$param3='INNER')
    {
        $this->join .= $param3.' JOIN '.$param1.' ON '.$param2.' ';
        return $this;
    }


    //添加表别名
    public function alias($param)
    {
        $this->alias = $param.' ';
        return $this;
    }

    //field字段
    public function field($_field)
    {
        $this->field = $_field;
        return $this;
    }

    //limit 数据条数
    public function limit($param1,$param2='')
    {
        empty($param2)?null:$param2 = ','.$param2; //当存在第二个参数时
        if(is_int($param1)){
            $this->limit = ' LIMIT '.$param1.$param2;
        }
        return $this;
    }

    //order 数据排序
    public function order($param)
    {
        empty($param)? null : $this->order= ' ORDER BY '.$param;
        return $this;
    }


    //判断变量类型，如果是字符串，内容加上单引号
    public function varType($var)
    {
        if(is_array($var)){
            echo 'Error:数据类型必须为数字或字符串';
        }
        if(!is_numeric($var)){ //如果是字符串
            return "'".$var."'";
        }else{
            return $var;
        }
    }



    /*
     * 原生查询
     * 新增、更新、删除操作
     * @return number 返回受影响的记录，如果新增还返回新增主键id
     */
    public function query($sql)
    {

        $result = $this->conn->prepare($sql);  //准备查询语句
        $res = $result->execute();  //执行查询语句，并返回结果集
        return $res;
    }

    //获取单条查询结果
    public function fetch($sql)
    {

        //如果开启了缓存
        if($this->cacheOption['switch']){
            if($this->cacheData != null){
                return $this->cacheData;
            }
        }

        $result = $this->conn->prepare($sql);  //准备查询语句
        $result->execute();  //执行查询语句，并返回结果集
        $res = $result->fetch(PDO::FETCH_ASSOC);
        $this->conn =null;

        //如果开启了缓存
        if($this->cacheOption['switch']){
            $cache = new Cache();
            $cache::set($this->cacheOption['name'],$res,$this->cacheOption['time']);
        }

        return $res;
    }

    //获取多条查询结果
    public function fetchAll($sql)
    {
        //dump($sql);
        //如果开启了缓存
        if($this->cacheOption['switch']){
            if($this->cacheData != null){
                return $this->cacheData;
            }
        }

        $result = $this->conn->prepare($sql);  //准备查询语句
        $result->execute();  //执行查询语句，并返回结果集
        $res =  $result->fetchAll(PDO::FETCH_ASSOC);

        //如果开启了缓存
        if($this->cacheOption['switch']){
            $cache = new Cache();
            $cache::set($this->cacheOption['name'],$res,$this->cacheOption['time']);
        }

        return $res;
    }


    //统计
    public function rowcount($sql)
    {
		 
        //如果开启了缓存
        if($this->cacheOption['switch']){
            if($this->cacheData != null){
                return $this->cacheData;
            }
        }

        $result = $this->conn->prepare($sql);  //准备查询语句
        $result->execute();  //执行查询语句，并返回结果集
        $res= $result->rowcount();


        //如果开启了缓存
        if($this->cacheOption['switch']){
            $cache = new Cache();
            $cache::set($this->cacheOption['name'],$res,$this->cacheOption['time']);
        }

        return $res;
    }

    /*
     * count 统计
     * @param string sql语句
     * @return number
     */
    public function count($param=''){
        if(empty($param)){
            $sql =  'SELECT '.$this->field.' FROM '.$this->table.$this->alias.$this->join.$this->where.$this->order.$this->limit;
        }else{
            $sql = $param;
        }
		 
        return $this->rowcount($sql);
    }


    /*
     * select 多条数据查询
     * @param string sql语句
     * @return array
     */
    public function select($param='')
    {
        if(empty($param)){
            $sql =  'SELECT '.$this->field.' FROM '.$this->table.$this->alias.$this->join.$this->where.$this->order.$this->limit;
        }else{
            $sql = $param;
        }
        return $this->fetchAll($sql);
    }


    /*
    * find单条数据查询
    * @param string sql语句
    * @return array
    */
    public function find($param='')
    {
        if(empty($param)){
            $sql =  'SELECT '.$this->field.' FROM '.$this->table.$this->where .' limit 1';
        }else{
            $sql = $param;
        }
        return $this->fetch($sql);
    }


    /*  paginate 分页
     *  @param int $param 分页数数
     *  @param array $config 分页配置
     *  @return object
     */
    public function paginate($param ='',$config='')
    {
        empty($_GET['page'])? $_page = 1: $_page = $_GET['page'];
        if($param !=''){
            $startNum = ($_page-1)*$param;
            $endNum = $param;
            $pagesize = ' LIMIT '.$startNum.','.$endNum;
        }else{
            echo 'ERROR: 分页参数不能为空！';
        }

        $pageDataSql =  'SELECT '.$this->field.' FROM '.$this->table.$this->alias.$this->join.$this->where.$this->order.$pagesize;
        $countSql = 'SELECT '.$this->field.' FROM '.$this->table.$this->alias.$this->join.$this->where;

		  
        $_pageData = $this->conn->query($pageDataSql)->fetchAll(PDO::FETCH_ASSOC);
        $_count = $this->count($countSql);

        $_page = new Paginate();
        $res = $_page->paginate($param,$config,$_pageData,$_count);
        return $res;
    }


    /*
     * 删除方法
     * $param  int|array 主键id
     */
    public function delete($param ='')
    {
        //不存在where条件
        if(empty($this->where)){
            //是否存在$param参数
            if(isset($param)){
                //$param为数组
                if(is_array($param)){
                    $param = implode(',',$param);
                    $sql = 'DELETE FROM '.$this->table.' WHERE id in('.$param.')';

                }else{//$param不是数组
                    $sql = 'DELETE FROM '.$this->table.' WHERE id='.$param;
                }
            }else{
                echo 'ERROR: 缺少主键';
            }
        }else{//存在where条件
            $sql = 'DELETE FROM '.$this->table.$this->where;
        }
        //执行删除并反回影响行数
        return $this->query($sql);
    }


    /*
     * 新增单条数据
     * $param  array 新增的数据的数组
     */
    public function insert($param)
    {
        $field = implode(',',array_keys($param)); //所有字段名
        $values = array();
        foreach (array_values($param) as $k=>$v){
            $values[$k] = $this->varType($v);
        }
        $values = implode(',',$values); //所有字段值

        $sql = "INSERT INTO ".$this->table."(".$field.") VALUES (".$values.")";
        //dump($sql);
        return $this->query($sql);

    }

    /*
     * 新增并返回新增id
     * $param  array 新增的数据的数组
     */
    public function insertGetId($param)
    {
        $field = implode(',',array_keys($param)); //所有字段名

        $values = array();
        foreach (array_values($param) as $k=>$v){
            $values[$k] = $this->varType($v);
        }
        $values = implode(',',$values); //所有字段值
        $sql = "INSERT INTO ".$this->table."(".$field.") VALUES (".$values.")";
        $this->query($sql);

        return $this->conn->lastInsertId();
    }

    /*
     * 新增单多数据
     * $param  array 新增的数据的二数组
     */
    public function insertAll($param)
    {
        $row = 0;
        foreach ($param as $k=>$v){
            $this->insert($v);
            $row++;
        }
        return $row;
    }


    //获取最后id
    public function getLastInsID()
    {
        return $this->max('id');
    }


    /*
     * 获取某一字段名的最大值
     * $param  string 字段名
     */
    public function max($param='')
    {
        if(!empty($param)){
            $sql = "select max(".$param.") FROM ".$this->table;
            $res =  $this->fetch($sql);
            return $res['max(id)'];
        }else{
            echo '字段名不能为空';
        }
    }


    /*
     * 更新方法
     * $param1  array|string 修改数据数组或字段名
     * $param2  string 对应$param1字段值
     */
    public function update($param1,$param2='')
    {
        if(empty($param1)){
            echo 'Error:更新参数不正确';
        }
        //当where更新条件不存在时，默认以id为更新条件
        if(empty($this->where)){
             isset($param1['id']) ? $this->where = 'WHERE id='.$param1['id'].' ' : exit('Error:缺少where更新条件');
        }
        //当存在参数$param2时
        if(!empty($param2)){
            if(is_array($param1)){
                exit('Error:参数1必须为字符串');
            }
            $UPDATE = $param1.'='.$this->varType($param2);
        }else{
            if(!is_array($param1)){
                exit('Error:参数1必须为数组');
            }
            $UPDATE = array();
            foreach ($param1 as $k=>$v) {
                $UPDATE[] = $k.'='.$this->varType($v);
            }
            $UPDATE = implode(',',$UPDATE);
        }

        $sql = 'UPDATE '.$this->table.' set '.$UPDATE.$this->where.$this->order.$this->limit;
        return $this->query($sql);
    }

    /*
     * 缓存方法
     * $param1  string 缓存标识
     * $param2  number 缓存时间
     */
    public function cache($name= '',$time= 0)
    {

        //启动并配置缓存
        if($name == ''){
            $name = md5($this->conn);
        }
        if($time == 0 && is_numeric($name)){ //如果只传了一个整数，判断为缓存时间
            $time = $name;
            $name = md5($this->table.$this->where.$this->field.$this->limit.$this->order.$this->join);
        }
        $this->cacheOption = array('switch'=>true,'name'=>$name,'time'=>$time);

        //获取缓存数据
        $cache = new Cache();
        $this->cacheData = $cache::get($name);
        return $this;
    }


}